@@ -15,12 +15,18 @@
 /*	to enable. */
 #define SERIAL_CONSOLE_DEFAULT 0
 
+/* SERIAL_TTY - The default serial port to use. 0=ttyS0, 1=ttyS1 */ 
+#define SERIAL_TTY 0
+
 /* SERIAL_BAUD_RATE - Baud rate for the serial console */
 #define SERIAL_BAUD_RATE 9600
 
 /* BEEP_MODE - Beep on error. Default off, Change to 1 to enable */
 #define BEEP_MODE 0
 
+/* BEEP_END_NO_ERROR - Beep at end of each pass without error. Default off, Change to 1 to enable */
+#define BEEP_END_NO_ERROR 0
+
 /* SCRN_DEBUG - extra check for SCREEN_BUFFER
  */ 
 /* #define SCRN_DEBUG */
